%--------------------------------------------------------------------------     
%  Mathematische Modellierung mit MATLAB, F. Hausser und Y. Luchko
%  
%  Basketball Freiwurf,Listing 2.4 und Listing 2.5: 
%     Bestimmung der maximal moeglichen Abweichung des Abwurfwinkels
%     von einem Abwurfwinkel theta0 bei fester Abwurfgeschindigkeit v0,
%     die so gewhlt ist, dass der Wurf mit theta0 durch das Zentrum des
%     Rings geht. Es wird nur Schwerkraft betrachtet, also Wurfparabeln
%     als Trajektorien.
%     
%  function fehler,thetaMin,thetaMax = fehlerWinkel(theta0Grad,hS)
%     Eingabeparameter:
%        theta0:     Referenz-Abwurfwinkel [grad]
%        hS:         Groesse des Werfers   [m]
%     Ausgabeparameter:
%        thetaMin:   minimaler Abwurfwinkel, so dass Ball noch im Korb
%        thetaMax:   maximaler Abwurfwinkel, so dass Ball noch im Korb
%        fehler:     min(thetaMax - theta0, theta0 - thetaMin)
%
%  Hilfsfunktion (Listing 2.5):
%   function zulaessig = winkelZulaessig(thetaGrad)
%
%     Author: Frank Hausser
%     Date:   8/2009
%--------------------------------------------------------------------------


function [fehler,thetaMin,thetaMax] = fehlerWinkel(theta0Grad,hS)

%--------------------  Parameter  -----------------------------------------
%   werden von der Funktion winkelZulaessig benoetigt 
if ( nargin < 2)
    hS = 2.13; % Dirk Nowitzky
end

parameter = getParameter();
l  = parameter.abstandHorizontal;          % Horizontale Entfernung [m]
h  = parameter.hoeheKorb - 5/4 * hS;       % vertikale Entfernung [m]
Rr = parameter.radiusRing;                 % Ringradius
Rb = parameter.radiusBall;                 % Ballradius
g  = parameter.erdbeschleunigung;          % Erdbeschleunigung

theta0 = theta0Grad*pi/180;
v0     = ( l/cos(theta0) ) * sqrt(  g/( 2*(l*tan(theta0) - h) )  );


%-------------------  Abbruch, falls theta0 nicht zulaessig ---------------

thetaGrad = theta0Grad;
if winkelZulaessig(thetaGrad) == false
    thetaMin = 0.;
    thetaMax = 0.;
    fehler   = 0.;
    return;
end

%---------- Bestimmung von thetaMin ---------------------------------------

% suche ersten unzulaessigen Winkel auf grobem Raster
thetaMin = theta0Grad - 0.5;
while( winkelZulaessig(thetaMin)  )
    thetaMin = thetaMin - 0.5;
end

% Bisektion zur Feinbestimmung
links  = thetaMin;
rechts = links + 0.5;
while abs(rechts - links) > 1.e-8
    mitte = (rechts + links)/2;
    if( winkelZulaessig(mitte) )
        rechts = mitte;
    else
        links = mitte;
    end
end
thetaMin = mitte;

%---------- Bestimmung von thetaMax ---------------------------------------

% suche ersten unzulaessigen Winkel auf grobem Raster
thetaMax = theta0Grad + 0.5;
while( winkelZulaessig(thetaMax)  )
    thetaMax = thetaMax + 0.5;
end

% Bisektion zur Feinbestimmung
rechts = thetaMax;
links  = thetaMax - 0.5;
while abs(rechts - links) > 1.e-8
    mitte = (rechts + links)/2;
    if( winkelZulaessig(mitte) )
        links = mitte;
    else
        rechts = mitte;
    end
end
thetaMax = mitte;

fehler = min( theta0Grad - thetaMin, thetaMax - theta0Grad );

%---------- Hilfsfunktion (nested function)  ------------------------------
%   Test ob ein Abwurfwinkel noch zu erfolgreichem Korb fuehrt

    function zulaessig = winkelZulaessig(thetaGrad)
        theta = thetaGrad*pi/180;
        vx0 = v0 * cos(theta);
        vy0 = v0 * sin(theta);
        % Schranken fuer zu untersuchende Zeitspanne
        T = (vy0 + sqrt(vy0*vy0 - 2*g*h))/g; % Zeit bis Ringhoehe in vertikaler
                                             % Richtung von oben erreicht
        t0 = (l - Rr - Rb)/vx0; % Zeit bis der Ballmittelpunkt in horizontaler
                                % Richtung nur noch den Abstand Radius Ball hat

        % Berechne Trajektorie (Wurfparabel) analytisch
        t = linspace(t0,T,100);
        x = vx0*t; y = vy0*t - g*t.*t/2;

        zulaessig = true;

        % Teste, ob Ball den Ring berhrt
        if( x(end) + Rb > l + Rr )      % hinterer Rand des Rings
            zulaessig = false;
        elseif ( x(end) - Rb < l - Rr ) % vorderer Rand des Rings
            zulaessig = false;         
        else                            % vorderer Rand des Rings
            xa = x - l + Rr;  % horizontaler Abstand des Ballmittelpunktes
            ya = y - h;       % vertikaler Abstand des Ballmittelpunktes
            a2 = xa.*xa + ya.*ya;
            if min(a2) < Rb*Rb
                zulaessig = false;
            end
        end
    end

end