%--------------------------------------------------------------------------   
%   Mathematische Modellierung mit MATLAB, F. Hausser und Y. Luchko
%  
%   Basketball Freiwurf, Listing 2.8, 2.7:
%         Bestimmung des zulaessigen Gebietes
%         fuer Abwurfwinkel theta^0 und Abwurfgeschwindigkeit v^0 
%     
%   function  [theta0Grad,v0Vorder,v0Center,v0Rueck] =
%                zulaessigesGebiet(thetaMin,thetaMax,hSpieler,info)
%
%   Eingabeparameter:
%        thetaMin:  minimaler Abwurfwinkel [grad]
%        thetaMax:  maximaler Abwurfwinkel [grad]
%        hS:        Groesse des Spielers   [m]
%
%   Ausgabeparameter:
%        theta0Grad: diskretisierter Definitionsbereich des Abwurfwinkels;
%                    [grad]; 
%        v0Vorne:    Abwurfgschwindigkeiten, bei denen Ball die Vorderseite
%                    des Ringes (von innen) streift
%        vZentrum:   Abwurfgeschwindigkeiten, bei denen Ball durch den
%                    Mittelpunkt des Ringes geht
%        vHinten:    Abwurfgeschwindigkeiten, bei denen Ball die
%                    Hinterseite des Ringes in Ringhoehe (von innen) trifft
%
%    Hilfsfunktion (Listing 2.7)
%    function ab = abstandMin(v,theta) 
%
%     Author: Frank Hausser
%     Date:   8/2009
%
%--------------------------------------------------------------------------



function [theta0Grad,v0Vorne,v0Zentrum,v0Hinten] ...  
                  = zulaessigesGebiet(thetaMin,thetaMax,hS)
parameter = getParameter();
l  = parameter.abstandHorizontal;  
h  = parameter.hoeheKorb - 5/4 * hS; 
Rr = parameter.radiusRing;         
Rb = parameter.radiusBall;   
g  = parameter.erdbeschleunigung;  

nTheta     = 100;  % Anzahl der diskreten Abwurfwinkel 
theta0Grad = linspace(thetaMin,thetaMax,nTheta);
theta0     = theta0Grad*pi/180;
vStart  = 6.5;

for k=1:nTheta
    theta = theta0(k); 
    % - - - Ball geht durch das Zentrum des Rings - - - - - - 
    v0Zentrum(k) = (l/cos(theta))*sqrt(g/(2*(l*tan(theta)-h)));

    % - - - Ball beruehrt den Ring hinten (auf Hoehe h) - - - 
    xT           = l + Rr - Rb;
    v0Hinten(k)  = (xT/cos(theta))*sqrt(g/(2*(xT*tan(theta)-h)));
  
    % - - - Ball beruehrt den Ring vorne (mit NB x(T)>=l-Rr) 
    [v0Vorne(k)] = fminsearch(@(v) abstandMin(v,theta),vStart);
    vStart       = v0Vorne(k);  %letztes Optimum als Startwert
end

% - - - - - - - - - Hilfsfunktion abstandMin - - - - - - - 
    function ab = abstandMin(v,theta)
        vx = v * cos(theta);
        vy = v * sin(theta);
        
        t0 = (l - Rr)/vx;                 % Anfang Zeitspanne
        T = (vy + sqrt(vy*vy - 2*g*h))/g; % Ende Zeitspanne 
        t = linspace(t0,T,100);
        x = vx*t; y = vy*t - g*t.*t/2;  % Wurfbahn, 100 Punkte

        xa = x - l + Rr;  % horizontaler Abstand 
        ya = y - h;       % vertikaler Abstand
        a2 = xa.*xa + ya.*ya; 
        ab = abs( sqrt(min(a2)) - Rb );     
       
        % Strafterm, realisiert die Nebenbedingung  x(T) >= l-Rr, 
        if( l - Rr - x(end) > 0)
            ab = ab + (l - Rr - x(end));
        end
    end
end





