%--------------------------------------------------------------------------   
%  Mathematische Modellierung mit MATLAB, F. Hausser und Y. Luchko
%  
%   Lsung der 3D-Transportglichung (6.20) nach dem
%   impliziten FD-Verfahren. 
%   Zuerst wird im Skript die Funktion sparseMatrixFD3D aufgerufen,
%   um die Koeffizientenmatrix A des linearen Gleichungssystems (6.24)
%   als eine sparse-Matrix zu erzeugen. Da das Gleichungssystems 
%   (6.24) in jedem Zeitschritt mit der gleichen Matrix und 
%   unterschiedlichen rechten Seiten gelst wird, wird nur einmal 
%   auerhalb der zeitlichen Schleife eine unvollstndige LU-Zerlegung der
%   Matrix $A$ bestimmt. Das lineare Gleichungssystem wird dann in 
%   jedem Zeitschritt mit einem iterativen Verfahren mit 
%   Vorkonditionierung gelst, wobei hier die unvollstndige 
%   LU-Zerlegung benutzt wird.
%   Die Ergebnisse der Berechnungen in allen rumlichen und zeitlichen 
%   Gitterpunkten werden in einer vierdimensionalen Matrix gespeichert und
%   mit dem Befehl scatter3 visualisiert.
%     Authoren: Peter Otto und Witalij Wambold
%     Date:   1/2010
%
%--------------------------------------------------------------------------


clear all;
close all;
% Systemparameter und Modellparameter
L = 1.0; W = 0.5; T = 0.5;
Tend = 10; dt = 0.05; maxk = Tend/dt;

dx = 0.1; dy = 0.05; dz = 0.05;
nx = L/dx; ny = W/dy; nz = T/dz;
% Diffusionskoeffizient
disp = 0.005;
% Geschwindigkeit der Konvektion	
vel = 4*[ 0.1 0.005 0.01];   
% Abbaurate
dec = .001;
% Rumliche Gitterpunkte
[y,x,z] = meshgrid(0:dy:W, 0:dx:L, 0:dz:T);
x1 = x(:); y1 = y(:); z1 = z(:);
% Speicherallokation fr die unbekannten Konzentrationen
u = zeros(nx+1,ny+1,nz+1,maxk+1);
% Randbedingungen 
transl = nx/2-1;
for k=1:maxk/4 
        u(1,ny/2,2,k) = 10;
        u(1,ny/2,3,k) = 10;
        u(1,ny/2+1,2,k) = 10;
        u(1,ny/2+1,3,k) = 10;
end
% Koeffizienten des Gleichungssystems
rdx2 = 1./(dx^2); rdy2 = 1./(dy^2); rdz2 = 1./(dz^2);
central_value = 1 + dt*disp*2*(rdx2+rdy2+rdz2) + vel(1)*dt/dx + vel(2)*dt/dy + vel(3)*dt/dz + dt*dec;
x_negative = -dt*(disp*rdx2 + vel(1)/dx); 
x_positive = -dt*disp*rdx2; 
y_negative = -dt*(disp*rdy2 + vel(2)/dy); 
y_positive = -dt*disp*rdy2; 
z_negative = -dt*(disp*rdz2 + vel(3)/dz);
z_positive = -dt*disp*rdz2;
% Koeffizientenmatrix A wird erzeugt
A = sparseMatrixFD3D( nx-1, ny-1, nz-1, central_value, x_negative, x_positive, y_negative, y_positive, z_negative, z_positive );
% unvollstndige LU-Zerlegung der Koeffizientenmatrix A
setup.type = 'crout';
setup.milu = 'row';
setup.droptol = 0.1;
[LM,UM] = ilu(A,setup);
% Die rechte Seite b_k des Gleichungssystems
for k=1:maxk    
    boundary = zeros(nx-1,ny-1,nz-1);
    boundary(1:nx-1, 1:ny-1, 1) = z_negative*u(2:nx, 2:ny, 1, k); 
    boundary(1:nx-1, 1:ny-1, nz-1) = z_positive*u(2:nx, 2:ny, nz+1, k); 
    boundary(1,    1:ny-1, 1:nz-1)  = boundary(1,    1:ny-1, 1:nz-1) + x_negative*u(1,    2:ny, 2:nz, k); 
    boundary(nx-1, 1:ny-1, 1:nz-1)  = boundary(nx-1, 1:ny-1, 1:nz-1) + x_positive*u(nx+1, 2:ny, 2:nz, k); 
    boundary(1:nx-1, 1,    1:nz-1)  = boundary(1:nx-1, 1,    1:nz-1) + y_negative*u(2:nx, 1,    2:nz, k); 
    boundary(1:nx-1, ny-1, 1:nz-1)  = boundary(1:nx-1, ny-1, 1:nz-1) + y_positive*u(2:nx, ny+1, 2:nz, k); 
    b = u(2:nx,2:ny,2:nz,k) - boundary;
    % Das Gleichungssystem (6.24) A U_k = b_k wird gelst  
    solution = bicgstab(A,b(:),1.e-5,100,LM,UM); 
    % Lsung wird gespeichert
    u(2:nx,2:ny,2:nz,k+1) = reshape(solution,nx-1,ny-1,nz-1);
end
% Die Ergebnisse werden visualisiert
scrsz = get(0,'ScreenSize');
figure('Position',[scrsz(1)+10 scrsz(2)+100 0.9*scrsz(3) 0.75*scrsz(4)]);
for k=1:maxk +1
    % tic;
    uk1 = u(:,:,:,k);
    v1 = uk1(:);
    nonzeros = find(v1 > 0.05);
    scatter3(x1(nonzeros),y1(nonzeros),z1(nonzeros),30*v1(nonzeros),v1(nonzeros),'filled');
    set(gca,'DataAspectRatio',[1 1 1],...
        'PlotBoxAspectRatio',[1 1 1],...
        'XLim',[0 L],'YLim',[0 W],'ZLim',[0 T],...
        'Box','on');
    cameraPos = get(gca,'CameraPosition');
    set(gca,'CameraPositionMode','manual',...
        'CameraPosition',[cameraPos(1), cameraPos(2), 0.3*cameraPos(3)]); 
    xlabel('x');
    ylabel('y');
    zlabel('z');
    shg;
   %waitforbuttonpress;
end