%--------------------------------------------------------------------------   
%  Mathematische Modellierung mit MATLAB, F. Hausser und Y. Luchko
%  
%   Lsung der 2D-Transportglichung (9.9) nach dem
%   impliziten FD-Verfahren. 
%   Zuerst wird im Skript die Funktion sparseMatrixFD3D aufgerufen,
%   um die Koeffizientenmatrix A des entsprechenden linearen 
%   Gleichungssystems als eine sparse-Matrix zu erzeugen. Dann wird 
%   auerhalb der zeitlichen Schleife eine unvollstndige LU-Zerlegung der
%   Matrix A bestimmt. Das lineare Gleichungssystem wird dann in 
%   jedem Zeitschritt mit einem iterativen Verfahren mit 
%   Vorkonditionierung gelst, wobei hier die unvollstndige 
%   LU-Zerlegung benutzt wird.
%   Die Ergebnisse der Berechnungen in allen rumlichen und zeitlichen 
%   Gitterpunkten werden in einer dreidimensionalen Matrix gespeichert und
%   mit dem Befehl mesh visualisiert.
%     Authoren: Peter Otto und Witalij Wambold
%     Date:   1/2010
%
%--------------------------------------------------------------------------

clear all;
close all;
% Systemparameter und Modellparameter 
T = 30.;  dt = 0.02; maxt = T/dt; 
L = 1.0; n = 50; h = L/n;
m = (n-1)*(n-1);  
% Diffusionskoeffizient
d = 0.0001;
% Abbaurate
dec = 0.01;
% Geschwindigkeit der Konvektion
convection = 0.1;
% Strmungsrichtung
alpha = atan(4); 
vx = convection*cos(alpha);  
vy = convection*sin(alpha);  
% Rumliche Gitterpunkte
x       = linspace(0,1,n+1);
y       = linspace(0,1,n+1);
[yi,xi] = meshgrid(y,x);
% Koeffizienten des Gleichungssystems
central_value = 4*d*dt/(h^2) + 1 + dec*dt + vx*dt/h + vy*dt/h;
x_negative = -d*dt/(h^2) - vx*dt/h; 
x_positive = -d*dt/(h^2); 
y_negative = -d*dt/(h^2) - vy*dt/h; 
y_positive = -d*dt/(h^2); 
% Koeffizientenmatrix A wird erzeugt
A = sparseMatrixFD3D( n-1, n-1, 1, central_value, x_negative, x_positive, y_negative, y_positive, 1, 1 );
% unvollstndige LU-Zerlegung der Koeffizientenmatrix A
setup.type = 'crout';
setup.milu = 'row';
setup.droptol = 0.1;
[LM,UM] = ilu(A,setup);
% Speicherallokation fr die unbekannten Konzentrationen
u = zeros(n+1,n+1,maxt+1); 
% Randbedingung
source_length = 0.2;
source_time = round(maxt*0.5);
sourse_size=round(source_length/h);
for k=1:source_time
   for j=1:sourse_size
       u(1,j,k) = 5*(-(1/((source_length*2)^2))*(j*h)^2+1);
       u(j,1,k) = u(1,j,k);
   end
end
% Die rechte Seite b_k des Gleichungssystems
uk = zeros(m,1);
for k = 1:maxt
    boundary = zeros(n-1,n-1);
    boundary(1, :)   = x_negative*u(1, 2:n, k); 
    boundary(n-1,:)  = x_positive*u(n+1, 2:n, k); 
    boundary(:, 1)   = boundary(:, 1) + y_negative*u(2:n, 1, k); 
    boundary(:, n-1) = boundary(:, n-1) + y_positive*u(2:n, n+1, k); 
    b = reshape(u(2:n,2:n,k) - boundary,m,1);
    % Das Gleichungssystem A U_k = b_k wird gelst    
    solution = bicgstab(A,b,1.e-5,100,LM,UM); 
    % Lsung wird gespeichert
    u(2:n, 2:n, k+1) = reshape(solution,n-1,n-1);
end

% Die Ergebnisse werden visualisiert
scrsz = get(0,'ScreenSize');
figure('Position',[scrsz(1)+10 scrsz(2)+50 0.9*scrsz(3) 0.75*scrsz(4)]);
lim =[x(2) x(end-1) y(2) y(end-1) 0 7];
for k=1:maxt
    mesh(xi(2:(end-1),2:(end-1)),yi(2:(end-1),2:(end-1)),u(2:(end-1),2:(end-1),k));
    set(gca,'XLim',lim(1:2),'YLim',lim(3:4),'ZLim',lim(5:6));
    cameraPos = get(gca,'CameraPosition');    
    set(gca,'CameraPositionMode','manual',...
        'CameraPosition',[-1.5*cameraPos(1), cameraPos(2), 3*cameraPos(3)]);
    xlabel('x');
    ylabel('y');
    zlabel('u(x,y)');
    axis(lim);
    shg;
    %waitforbuttonpress;
end
